@@ -12,6 +12,9 @@
 #include "obs-websocket-api.h"
 #include "obs.h"
 
+typedef void (*get_transitions_callback_t)(
+	void *data, struct obs_frontend_source_list *sources);
+
 class LockedCheckBox : public QCheckBox {
 	Q_OBJECT
 
@@ -20,7 +23,7 @@ public:
 	explicit LockedCheckBox(QWidget *parent);
 };
 
-enum transitionType { match, show, hide };
+enum transitionType { match, show, hide, override };
 
 class DownstreamKeyer : public QWidget {
 	Q_OBJECT
@@ -30,16 +33,20 @@ private:
 	obs_source_t *transition;
 	obs_source_t *showTransition;
 	obs_source_t *hideTransition;
+	obs_source_t *overrideTransition;
 	QListWidget *scenesList;
 	QToolBar *scenesToolbar;
 	uint32_t transitionDuration;
 	uint32_t showTransitionDuration;
 	uint32_t hideTransitionDuration;
+	uint32_t overrideTransitionDuration;
 	LockedCheckBox *tie;
 	obs_hotkey_id null_hotkey_id;
 	obs_hotkey_pair_id tie_hotkey_id;
 	std::set<std::string> exclude_scenes;
-	obs_websocket_vendor vendor;
+	obs_view_t *view = nullptr;
+	get_transitions_callback_t get_transitions = nullptr;
+	void *get_transitions_data = nullptr;
 
 	static void source_rename(void *data, calldata_t *calldata);
 	static void source_remove(void *data, calldata_t *calldata);
@@ -70,7 +77,9 @@ private slots:
 signals:
 
 public:
-	DownstreamKeyer(int channel, QString name, obs_websocket_vendor vendor);
+	DownstreamKeyer(int channel, QString name, obs_view_t *view = nullptr,
+			get_transitions_callback_t get_transitions = nullptr,
+			void *get_transitions_data = nullptr);
 	~DownstreamKeyer();
 
 	void Save(obs_data_t *data);
@@ -90,4 +99,5 @@ public:
 	bool AddScene(QString scene_name);
 	bool RemoveScene(QString scene_name);
 	void SetTie(bool tie);
+	void SetOutputChannel(int outputChannel);
 };
